package jalview.rest;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jalview.bin.Cache;
import jalview.gui.Desktop;
import jalview.gui.SequenceFetcher;
import jalview.util.DBRefUtils;

public class FetchSequenceEndpoint extends EndpointAsync
{
  public FetchSequenceEndpoint(API api)
  {
    super(api, name);
  }

  protected static final String name = "fetchsequence";

  private SequenceFetcher sf;

  @Override
  protected void initialise(HttpServletRequest request,
          HttpServletResponse response)
  {
    String[] parameters = getEndpointPathParameters(request);

    // check we can run fetchsequence
    if (parameters.length < 2)
    {
      returnError(request, response,
              "requires 2 path parameters: dbname, ids");
      return;
    }

    String dbName = parameters[0];
    String dbId = parameters[1];

    setId(request, dbName + "::" + dbId);
    if (checkStatus(request, response))
      return;

    String db = DBRefUtils.getCanonicalName(dbName);
    Desktop desktop = Desktop.instance;
    sf = new SequenceFetcher(desktop, db, dbId);
    setCompletableFuture(sf.ok_actionPerformed(true));
  }

  protected void process(HttpServletRequest request,
          HttpServletResponse response, Map<String, String> map)
  {
    // all the work being done by the SequenceFetcher!
    Cache.warn("THIS SHOULD NOT BE RUN");
  }

  protected void atEnd()
  {
    sf.close_actionPerformed(null);
  }
}
